/**********************************************************************
 *
 * sqliteclass.cpp -- 
 * Copyright (C) 2008  DL Consulting Ltd
 *
 * A component of the Greenstone digital library software
 * from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *********************************************************************/

#include "sqliteclass.h"


#define SQLITE_MAX_RETRIES 8


sqliteclass::~sqliteclass()
{
  closedatabase();
}


// returns true if opened
bool sqliteclass::opendatabase(const text_t &filename, int mode, int num_retrys, 
#ifdef __WIN32__
			       bool need_filelock
#else
			       bool
#endif
			      )
{
  // Check if we've already got the database open
  if (sqlitefile != NULL)
  {
    if (openfile == filename) return true;
    else closedatabase();
  }

  char *filename_cstr = filename.getcstr();
  sqlitefile = NULL;
  sqlite3_open(filename_cstr, &sqlitefile);
  delete[] filename_cstr;

  if (sqlitefile == NULL && logout != NULL)
  {
    outconvertclass text_t2ascii;
    (*logout) << text_t2ascii << "database open failed on: " << filename << "\n";
  }

  return (sqlitefile != NULL);
}


void sqliteclass::closedatabase()
{
  if (sqlitefile == NULL) return;

  sqlite3_close(sqlitefile);
  sqlitefile = NULL;
  openfile.clear();
}


// returns true on success
bool sqliteclass::getinfo(const text_t& key, infodbclass &info)
{
  // !! TO IMPLEMENT
  return false;
}


// returns true if exists
bool sqliteclass::exists(const text_t& key)
{
  // !! TO IMPLEMENT
  return false;
}


// returns true on success
bool sqliteclass::setinfo(const text_t &key, const infodbclass &info)
{
  if (sqlitefile == NULL) return false;

  // !! TO IMPLEMENT
  return false;
}


void sqliteclass::deletekey(const text_t &key)
{
  if (sqlitefile == NULL) return;

  // !! TO IMPLEMENT
}


text_t sqliteclass::getfirstkey()
{
  if (sqlitefile == NULL) return g_EmptyText;

  // !! TO IMPLEMENT
  return g_EmptyText;
}


text_t sqliteclass::getnextkey(const text_t &key)
{
  if (sqlitefile == NULL || key.empty()) return g_EmptyText;

  // !! TO IMPLEMENT
  return g_EmptyText;
}


// sqlexec simply executes the given sql statement - it doesn't obtain a
// result set - returns true if the sql statement was executed successfully
// logerrors may be set to false to prevent this method from logging an
// error if it fails - this is useful in some circumstances when we execute
// an sql command specifically to see if it fails (e.g. when testing if a
// table exists in the db)
bool sqliteclass::sqlexec(const text_t &sql_cmd, bool logerrors)
{
  char *sql_cmd_cstr = sql_cmd.getcstr();

  int rv = 0;
  int tries = 0;
  while ((rv = sqlite3_exec(sqlitefile, sql_cmd_cstr, NULL, NULL, NULL)) == SQLITE_BUSY)
  {
    sleep(1000);
    tries++;
    if (tries > SQLITE_MAX_RETRIES)
    {
      outconvertclass text_t2ascii;
      (*logout) << text_t2ascii << "max_retries exceeded for sql query: " << sql_cmd << "\n";
      break;
    }
  }

  delete[] sql_cmd_cstr;

  if (rv == SQLITE_OK) return true;

  if (logerrors)
  {
    outconvertclass text_t2ascii;
    (*logout) << text_t2ascii << "Error executing sql statement: " << sql_cmd << "\n";
  }

  return false;
}


// sleep for the given number of milliseconds
void sleep(int m)
{
#ifdef __WIN32__
  Sleep(m);
#else
  usleep(m);
#endif
}
